/* C */
#include <stdio.h>
#include <ctype.h>

/* Windows */
#include <wtypes.h>
#include <wincon.h>

/* Local */
#include "winmutt.h"
#include "wmutil.h"
#include "wmvars.h"

/* skip over whitespace on a file */

int wm_eatWS(FILE *handle) {
  int result = 0, ch = '\0';

  while(!feof(handle) && isspace(ch = fgetc(handle)))
    result++;

  if(result) ungetc(ch, handle);

  return result;
}


/* wait for a keypress or mouse click */

void wm_getPress() {
  INPUT_RECORD inBuf;
  KEY_EVENT_RECORD kEvent;
  DWORD eventsRead;
  DWORD err;
  int hit = 0;

  FlushConsoleInputBuffer(wmConIn);

  while(!hit) {
    if(!ReadConsoleInput(wmConIn, &inBuf, 1, &eventsRead)) {
      err = GetLastError();
      printf("Oh, I can't read console input any more.  byebye. (%ld)\n", err);
      exit(1);
    }
    switch(inBuf.EventType) {
      case KEY_EVENT:
        kEvent = inBuf.Event.KeyEvent;
        if(kEvent.bKeyDown == 1) hit = 1;
        break;
      case MOUSE_EVENT: /* I'll let them click too */
        if(inBuf.Event.MouseEvent.dwEventFlags != MOUSE_MOVED &&
           inBuf.Event.MouseEvent.dwButtonState != MOUSE_WHEELED) /* ie, a (double)-click */
          if(inBuf.Event.MouseEvent.dwButtonState != 0) hit = 1;
        break;
    }
  }
}


/* get a 0-11 month number from the name */

const char *monthNames[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

int wm_getMonNo(char *month) {
  int i, mo = -1;

  for(i=0; i<12; i++) {
    if(!strnicmp(month, monthNames[i], 3)) mo = i;
  }

  return mo;
}


/* display line on bottom row of console */

void wm_showMessage(char *msg, WORD msgColour) {
  COORD newPos;
  DWORD written;
  char outBuf[512]; /* assumption again.  console's no wider than 512 */

  if(strlen(msg) > (unsigned)wmConSizeX) *(msg + wmConSizeX) = '\0';
  strcpy(outBuf, msg);
  /* set trailing bit to spaces */
  memset(outBuf + strlen(msg), ' ', wmConSizeX - strlen(msg));
  *(outBuf + wmConSizeX) = '\0';

  /* set colour, move, and write */
  SetConsoleTextAttribute(wmConOut, msgColour);
  newPos.X = 0; newPos.Y = (wmConSizeY) - 1;
  SetConsoleCursorPosition(wmConOut, newPos);
  WriteConsole(wmConOut, outBuf, wmConSizeX, &written, NULL);
}


/* show status or key-bar line */

void wm_showStatusOrKeys(char *msg, int whichLine) {
  COORD newPos;
  DWORD written;
  char outBuf[512]; /* assumption again.  console's no wider than 512 */

  if(strlen(msg) > (unsigned)wmConSizeX) *(msg + wmConSizeX) = '\0';
  strcpy(outBuf, msg);
  /* set trailing bit to spaces */
  memset(outBuf + strlen(msg), ' ', wmConSizeX - strlen(msg));
  *(outBuf + wmConSizeX) = '\0';

  /* set colour, move, and write */
  SetConsoleTextAttribute(wmConOut, wmColours[WMSTATUSCOL].colourNum);
  newPos.X = 0; newPos.Y = (whichLine == WMSTATUS ? (wmConSizeY) - 2 : 0);
  SetConsoleCursorPosition(wmConOut, newPos);
  WriteConsole(wmConOut, outBuf, wmConSizeX, &written, NULL);
}


/* display error message */

void wm_dispError(char *msg) {
  wm_showMessage(msg, wmColours[WMERRORCOL].colourNum);
}


/* output a line to the screen - for startup mainly */

void wm_writeString(char *msg) {
  CONSOLE_SCREEN_BUFFER_INFO csbi;
  DWORD written;
  COORD curPos;

  GetConsoleScreenBufferInfo(wmConOut,&csbi);
  curPos = csbi.dwCursorPosition;
  if(curPos.Y == 0) { /* don't use top line */
    curPos.Y = 1;
    SetConsoleCursorPosition(wmConOut, curPos);
  }

  if(!WriteConsoleOutputCharacter(wmConOut, msg, strlen(msg), curPos, &written)) {
    GetLastError();
    wm_dispError("Fatal: WriteConsoleOutputCharacter failed");

    wm_getPress();
    exit(1);
  }

  /* jump forward as many lines as appropriate */
  curPos.Y += (strlen(msg) / wmConSizeX) + 1;
  SetConsoleCursorPosition(wmConOut, curPos);
}


/* clear screen */

void wm_cls()
{
  DWORD numChars;
  DWORD written;
  COORD startCoord;

  SetConsoleTextAttribute(wmConOut, (WORD)(WMCOL_FGDEFAULT | WMCOL_BGDEFAULT));
  numChars = wmConSizeX * wmConSizeY;

  startCoord.X = 0; startCoord.Y = 0;
  FillConsoleOutputCharacter(wmConOut, ' ', numChars, startCoord, &written);
  FillConsoleOutputAttribute(wmConOut, (WORD)(WMCOL_FGDEFAULT | WMCOL_BGDEFAULT),
      numChars, startCoord, &written);
}


/* highlight a line on the screen */

void wm_setHighlight(int newLine) {
  COORD newPos;
  DWORD written;

  int arrowOpt = wm_findOpt("arrow_cursor")->optionValue.optionNumValue;

  /* remove the old one */
  if(wmCurrLine != -1) {
    newPos.X = 0; newPos.Y = wmCurrLine;

    if(arrowOpt == WMOPTVAL_YES) {
      SetConsoleTextAttribute(wmConOut, wmColours[WMNORMALCOL].colourNum);
      SetConsoleCursorPosition(wmConOut, newPos);
      WriteConsole(wmConOut, "  ", 2, &written, NULL);
    } else {
      FillConsoleOutputAttribute(wmConOut, wmColours[WMNORMALCOL].colourNum, wmConSizeX, newPos, &written);
    }
  }

  /* and put in the new one */
  newPos.X = 0; newPos.Y = newLine;

  if(arrowOpt == WMOPTVAL_YES) {
    SetConsoleTextAttribute(wmConOut, wmColours[WMINDICATORCOL].colourNum);
    SetConsoleCursorPosition(wmConOut, newPos);
    WriteConsole(wmConOut, "->", 2, &written, NULL);
  } else {
    FillConsoleOutputAttribute(wmConOut, wmColours[WMINDICATORCOL].colourNum, wmConSizeX, newPos, &written);
  }

  wmCurrLine = newLine;
}
